/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 08 - Esercizio 08 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <math.h>

#define N     10
#define DIM  N*N

/* function prototypes */
int readMatrix(int [][DIM]);
int blockError(int [][DIM], int, int, int);

/*
 *  main program
 */
int main(void)
{
  int matrix[DIM][DIM], dim;
  int r, c, n, block, error;

  dim = readMatrix(matrix);
  n = floor(sqrt(dim));
  block = error = 0;

  for (r=0; r<dim && !error; r+=n) {
    for (c=0; c<dim && !error; c+=n) {
      block++;
      if (blockError(matrix, r, c, n)) {
        error = 1;
      }
    }
  }

  if (error) {
    printf("NOT valid matrix: error in block %d.\n", block);
  } else {
    printf("Valid matrix.\n");
  }
  return 0;
} 

/*
 *  read in the matrix
 */
int readMatrix(int matrix[DIM][DIM])
{
  int row, rowMax, col, colMax, val;
  char separator;

  /* init the matrix */
  for (row=0; row<DIM; row++) {
    for (col=0; col<DIM; col++) {
      matrix[row][col] = 0;
    }
  }

  /* read the matrix */
  row = col = 0;
  colMax = rowMax = DIM;
  printf("Input the square matrix:\n");
  do {
    scanf("%d%c", &val, &separator);
    if (col < colMax) {
      matrix[row][col] = val;
      col++;
    }
    if (separator == '\n') {
      colMax = rowMax = col;
      col = 0;
      row++;
    }
  } while (row < rowMax);
  return row;
}

/*
 *  check for an error in a block
 */
int blockError(int matrix[][DIM], int row, int col, int n)
{
  int r, c, i, j, error=0;

  for (r=0; r<n && !error; r++) {
    for (c=0; c<n && !error; c++) {
      /* check for values out of bounds */
      if (matrix[row+r][col+c]<1 || matrix[row+r][col+c]>n*n) {
        error = 1;
      }

      /* check for duplicates */
      for (i=r; i<n && !error; i++) {
        for (j=((i==r)?(c+1):0); j<n && !error; j++) {
          if (matrix[row+i][col+j] == matrix[row+r][col+c]) {
            error = 1;
          }
        }
      }
    }
  }
  return error;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
