/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 08 - Esercizio 10 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>

#define DIM1     5
#define DIM2     5

/* function prototypes */
void update(char [][DIM1][DIM2], int);
void reset(char [][DIM1][DIM2], int);
int alive(char [][DIM1][DIM2], int, int, int);
int neighbour(char [][DIM1][DIM2], int, int, int);
void display(char [][DIM1][DIM2], int);

/*
 *  main program
 */
int main(void)
{
  char matrix[2][DIM1][DIM2], goOn;
  int k, x, y, flag=0;

  /* init matrix */
  for (k=0; k<2; k++) {
    reset(matrix, k);
  }

  printf("Input:\n");
  printf("  RowIndex<space>ColumnIndex<return>\n");
  printf("  RowIndex = 0..%d\n", DIM1-1);
  printf("  ColumnIndex = 0..%d\n", DIM2-1);

  /* input matrix (initial configuration) */ 
  do {
    printf("RowIndex<space>ColumnIndex<return>: ");
    scanf("%d %d", &x, &y);
    if (x>=0 && x<DIM1 && y>=0 && y<DIM2) {
      matrix[flag][x][y] = '*';
    }
  } while (x>=0 || y>=0);

  display(matrix, 0);

  /* evolution */
  do {
    update(matrix, flag);
    display(matrix, !flag);
    flag = !flag;

    printf("Go on (1=yes, 0=no)? ");
    scanf("%*c%c", &goOn);
  } while (goOn == '1');
  return 0;
}

/*
 *  clear the matrix related to the old generation
 */
void reset(char matrix[2][DIM1][DIM2], int flag)
{
  int i, j;

  for (i=0; i<DIM1; i++) {
    for (j=0; j<DIM2; j++) {
      matrix[flag][i][j] = ' ';
    }
  }
}

/*
 *  compute the next generation
 */
void update(char matrix[2][DIM1][DIM2], int flag)
{
  int i, j, n;

  for (i=0; i<DIM1; i++) {
    for (j=0; j<DIM2; j++) {
      n = neighbour(matrix, i, j, flag);
      if (alive(matrix, i, j, flag)) {
        if (n==2 || n==3) {
          matrix[!flag][i][j] = '*';
        }
      } else {
        if (n == 3) {
          matrix[!flag][i][j] = '*';
        }
      }
    }
  }

  reset(matrix, flag);
}

/*
 *  count the number of alive neighbour elements 
 */
int neighbour(char matrix[2][DIM1][DIM2], int x, int y, int flag)
{
  int i, xx, yy, count=0;
  /* offset arrays */
  int offX[8] = {0, 1, 1, 1, 0, -1, -1, -1};
  int offY[8] = {1, 1, 0, -1, -1, -1, 0, 1};

  for (i=0; i<8; i++) {
    xx = x + offX[i];
    yy = y + offY[i];
    if (xx>=0 && xx<DIM1 && yy>=0 && yy<DIM2) {
      if (alive(matrix, xx, yy, flag)) {
        count++;
      }
    }
  }

  return count;
}

/*
 *  return "true" if the specified matrix element is alive
 */
int alive(char matrix[2][DIM1][DIM2], int x, int y, int flag)
{
  return (matrix[flag][x][y] == '*');
}

/*
 *  print a generation
 */
void display(char matrix[2][DIM1][DIM2], int flag)
{
  int i, j;

  for (i=0; i<DIM2+2; i++) {
    printf("#");
  }

  printf("\n");
  for (i=0; i<DIM1; i++) {
    printf("#");
    for (j=0; j<DIM2; j++) {
      printf("%c", matrix[flag][i][j]);
    }
    printf("#\n");
  }

  for (i=0; i<DIM2+2; i++) {
    printf("#");
  }
  printf("\n\n");
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
