/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 08 - Esercizio Proposto 04 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>

#define MAX  10   /* max number of elements in the list */

/* function prototypes */
int read(void);
int insert(int list[], int dim, int val);
void search(int list[], int dim, int val);
int deleteFirst(int list[], int dim);
int deleteLast(int list[], int dim);
int delete(int list[], int dim, int val);
void display(int list[], int dim);

/*
 *  main program
 */
int main(void)
{
  int list[MAX], index=0, val;
  char choice;

  do {
    printf("\nAvailable commands:\n");
    printf("  i: insert a value (sorted)\n");
    printf("  s: search a value\n");
    printf("  f: delete the first value\n");
    printf("  l: delete the last value\n");
    printf("  d: delete a specified value\n");
    printf("  c: display the list contents\n");
    printf("  e: end program\n");
    printf("Make your choice: ");
    scanf("%c%*c", &choice);

    switch (choice) {
      case 'i': val = read();
                index = insert(list, index, val);
                break;
      case 's': val = read();
                search(list, index, val);
                break; 
      case 'f': index = deleteFirst(list, index);
                break;
      case 'l': index = deleteLast(list, index);
                break;
      case 'd': val = read();
                index = delete(list, index, val);
                break;
      case 'c': display(list, index);
                break;
      case 'e': printf("End of session.\n");
                break;
      default : printf("Wrong choice!\n");
                break;
    }
  } while (choice != 'e');

  return 0;
}

/*
 *  read in a value
 */
int read(void)
{
  int val;

  printf("Value: ");
  scanf("%d%*c", &val);

  return val;
}

/*
 *  insert a value in the list (sorted)
 */
int insert(int list[], int dim, int val)
{
  int i=dim;

  if (dim >= MAX) {
    printf("The list is full, operation SKIPPED!\n");
    return dim;
  } 

  while (--i>=0 && val<list[i]) {
    list[i+1] = list[i];
  }
  list[i+1] = val;
  return dim+1;
}

/*
 *  search a value in the list
 */
void search(int list[], int dim, int val)
{
  int i;

  for (i=0; i<dim && list[i]<val; i++) ;

  if (i<dim && val==list[i]) {
    printf("Element found (index = %d)\n", i);
  } else {
    printf("Element NOT found.\n");
  }
}

/*
 *  delete the first element of the list
 */
int deleteFirst(int list[], int dim)
{
  int i;

  if (dim <= 0) {
    printf("The list is empty, operation SKIPPED!\n");
    return dim;
  } 

  for (i=1; i<dim; i++) {
    list[i-1] = list[i];
  }
  return dim-1;
}

/*
 *  delete the last element of the list
 */
int deleteLast(int list[], int dim)
{
  if (dim <= 0) {
    printf("The list is empty, operation SKIPPED!\n");
    return dim;
  } 

  return dim-1;
}

/*
 *  delete a list element, keeping it sorted
 */
int delete(int list[], int dim, int val)
{
  int i, j;

  for (i=0; i<dim && list[i]<val; i++) ;

  if (i<dim && val==list[i]) {
    for (j=i+1; j<dim; j++) {
      list[j-1] = list[j];
    }
    return dim-1;
  }

  printf("Element NOT found.\n");
  return dim;
}

/*
 *  display the list contents
 */
void display(int list[], int dim)
{
  int i;

  if (dim == 0) {
    printf("The list is empty\n");
  } else {
    for (i=0; i<dim; i++) {
      printf("Element %d = %d\n", i, list[i]);
    }
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
