/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 09 - Esercizio 13 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#define MAX 501

/*
 *  check whether the input char is a vocal
 */
int isvocal(char ch)
{
  char c = tolower(ch);

  return ((c=='a') || (c=='e') || (c=='i') || (c=='o') || (c=='u'));
}

/*
 *  encoding function
 */
void encode(char *message)
{
  int i, j, length=strlen(message);
  char save;

  /* save the last vocal */
  j = length-1;
  while (j>=0 && !isvocal(message[j])) {
    j--;
  }
  save = message[j];

  /* right shift all previous vocals */
  i = j-1;
  while (i >= 0) {
    while (i>=0 && !isvocal(message[i])) {
      i--;
    }
    if (i >= 0) {
      message[j] = message[i];
      j = i--;
    }
  }

  /* replace the first vocal */
  message[j] = save;
}

/*
 *  decoding function
 */
void decode(char *message)
{
  int i, j, length=strlen(message);
  char save;

  /* save the first vocal */
  j = 0;
  while (j<length && !isvocal(message[j])) {
    j++;
  }
  save = message[j];

  /* left shift all next vocals */
  i = j+1;
  while (i < length) {
    while (i<length && !isvocal(message[i])) {
      i++;
    }
    if (i < length) {
      message[j] = message[i];
      j = i++;
    }
  }

  /* replace the last vocal */
  message[j] = save;
}

/*
 *  main program
 */
int main(void)
{
  char message[MAX], cmd;
  int i=0;

  /* input phase */
  printf("Insert the string: ");
  do {
    scanf("%c", &message[i]);
  } while (message[i++] != '\n');
  message[i-1] = '\0';

  printf("Encode (E) or Decode (D)? ");
  scanf("%c", &cmd);

  /* computation and output */
  if (toupper(cmd) == 'E') {
    /* encoding */
    encode(message);
    printf("Encoded message is: %s\n", message);
  } else {
    /* decoding */
    decode(message);
    printf("Decoded message is: %s\n", message);
  }
  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
