/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio 08 - Soluzione 2
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#define MAX_WORD   20+1
#define MAX_LINE   100*20+1+1

/* function prototypes */
void replace(char *, char *, char *, char *);

int main(int argc, char *argv[])
{
  char lineSrc[MAX_LINE], lineDst[MAX_LINE];
  char wordSrc[MAX_WORD], wordDst[MAX_WORD];
  FILE *fin, *fout;
  int i;

  /* check command line */
  if (argc < 3) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <inputFile> <outputFile>\n", argv[0]);
    return 1;
  }

  /* open files */
  fin = fopen(argv[1], "r");
  fout = fopen(argv[2], "w");
  if (fin==NULL || fout==NULL) {
    printf("Error while opening the files.\n");
    return 1;
  }

  /* get the words to process */
  printf("Input the word to search: ");
  scanf("%s", wordSrc);
  printf("Input the word to replace: ");
  scanf("%s", wordDst);

  i = 0;
  while (fgets(lineSrc, MAX_LINE, fin) != NULL) {
    replace(lineSrc, wordSrc, wordDst, lineDst);
    fprintf(fout, "%s", lineDst);
  }
  fclose(fin);
  fclose(fout);
  return 0;
}

/*
 *  replace any (case insensitive) occurrence of "wordSrc" within "lineSrc"
 *  with "wordDst"; put the result in "lineDst".
 */
void replace(char *lineSrc, char *wordSrc, char *wordDst, char *lineDst)
{
  int srcIdx, dstIdx, j, flag;
  int sll, swl, dwl;
  
  sll = strlen(lineSrc);
  swl = strlen(wordSrc);
  dwl = strlen(wordDst);

  srcIdx = dstIdx = 0;
  while (srcIdx < sll) {
    flag = 0;
    j = 0;
    while (j<swl && flag==0) {
      if (srcIdx+j>=sll || toupper(lineSrc[srcIdx+j])!=toupper(wordSrc[j])) {
        flag = 1;
      }
      j++;
    }

    if (flag == 0) {
      /* wordSrc found: replace with wordDst in lineDst */
      for (j=0; j<dwl; j++) {
        lineDst[dstIdx+j] = wordDst[j];
      }
      srcIdx = srcIdx + swl;
      dstIdx = dstIdx + dwl;
    } else {
      /* wordSrc NOT found: copy one char of lineSrc in lineDst */
      lineDst[dstIdx] = lineSrc[srcIdx];
      srcIdx++;
      dstIdx++;
    }
  }

  lineDst[dstIdx] = '\0';
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
