/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio 09 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>

/* constants */
#define DIM   20
#define LEN   30
#define NR     5   /* number of rows displayed for each cell */
#define NC     6   /* number of columns displayed for each cell */

/* function prototypes */
int readSchema(char mat[DIM][DIM], int *nr, int *nc);
void separatorLine(int nc);
void full(void);
void empty(void);
void numbered(int count);
int isNumbered(char mat[DIM][DIM], int nr, int nc, int i, int j);

/*
 *  main program
 */
int main(void)
{
  char mat[DIM][DIM];
  int i, j, rip, nr, nc, count=1;

  if (readSchema(mat, &nr, &nc) == 1) {
    for (i=0; i<nr; i++) {
      separatorLine(nc);

      /* handle the i-th matrix row */
      for (rip=0; rip<NR; rip++) {
        for (j=0; j<nc; j++) {
          /* handle element [i, j] */
          printf("|");
          if (mat[i][j] == '1') {
            full();
          } else if (mat[i][j] == '0') {
            /* check on numbering */
            if (rip==0 && isNumbered(mat, nr, nc, i, j)) {
              numbered(count++);
            } else {
              empty();
            }
          } else {
            /* error */
            printf("Invalid matrix symbol (%c).\n", mat[i][j]);
            return 1;
          }
        } /* for j */
        printf("|\n");
      } /* for rip */
    } /* for i */
    separatorLine(nc);
  } 
  return 0;
}

/* 
 *  input schema: load matrix and compute its dimensions
 */
int readSchema(char mat[DIM][DIM], int *nr, int *nc)
{
  char name[LEN];
  char line[DIM+2];
  int i, j;
  FILE *fp;

  printf("Input file name: ");
  scanf("%s", name);
  fp = fopen(name, "r");
  if (fp == NULL) {
    printf("Error opening the input file.\n");
    return 0;
  }

  /* matrix load */
  i = 0;
  while (fgets(line, DIM+2, fp) != NULL) {
    j=0; 
    while (j<DIM && line[j]!='\n') {
      mat[i][j] = line[j];
      j++;
    } 
    i++;
  }
  fclose (fp);
  
  *nc = j; 
  *nr = i;
  return 1;
}

/* 
 *  display an entire line of '-'
 */
void separatorLine(int nc)
{
  int i, l;

  l = nc * (NC+1) + 1;
  for (i=0; i<l; i++) {
    printf("-");
  }
  printf("\n");
}

/* 
 *  display a row related to a "full" cell
 */
void full()
{
  int i;

  for (i=0; i<NC; i++) {
    printf("*");
  }
}

/* 
 *  display a row related to an "empty" cell
 */
void empty()
{
  int i;

  for (i=0; i<NC; i++) {
    printf(" ");
  }
}

/* 
 *  display a row related to a "numbered" cell
 */
void numbered(int count)
{
  int i;

  for (i=0; i<NC/2-1; i++) {
    printf(" ");
  }
  printf("%2d", count);
  for (i=0; i<NC/2-1; i++) {
    printf(" ");
  }
}

/* 
 *  check whether an empty cell needs a number
 */
int isNumbered(char mat[DIM][DIM], int nr, int nc, int i, int j)
{
  /* "vertical" test */
  if (i == 0) {
    if (mat[i+1][j] == '0') {
      return 1;
    }
  } else {
    if (i < nr-1) {
      if (mat[i-1][j] != '0' && mat[i+1][j] == '0') {
        return 1;
      }
    }
  }

  /* "horizontal" test */
  if (j == 0) {
    if (mat[i][j+1] == '0' ) {
      return 1;
    }
  } else {
    if (j < nc-1) {
      if (mat[i][j-1] != '0' && mat[i][j+1] == '0') {
        return 1;
      }
    }
  }

  /* neither vertical nor horizontal numbering */
  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
