/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio 10 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>

#define   DIM        30

/* function prototypes */
void find(char word[], char matrixIn[][DIM], char matrixOut[][DIM]);
int findAll(int row, int col, char word[], char mIn[][DIM], char mOut[][DIM]);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  char matrixIn[DIM][DIM], matrixOut[DIM][DIM];
  char word[DIM+1], c;
  int i, j;
  FILE *fp;

  /* check command line */
  if (argc < 4) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <matrixFile> <wordFile> <outputFile>\n", argv[0]);
    return 1;
  }

  /* initialization */
  for (i=0; i<DIM; i++) {
    for (j=0; j<DIM; j++) {
      matrixIn[i][j] = matrixOut[i][j] = ' ';
    }
  }

  /* load matrix */
  fp = fopen(argv[1], "r");
  if (fp == NULL) {
    printf("Error opening the matrix file.\n");
    return 1;
  }
  i = j = 0;
  while (fscanf(fp, "%c", &c) != EOF) {
    if (c != '\n') {
      matrixIn[i][j] = c;
      j++;
    } else {
      j=0;
      i++;
    }
  }
  fclose(fp);

  /* computation */  
  fp = fopen(argv[2], "r");
  if (fp == NULL) {
    printf("Error opening the words file.\n");
    return 1;
  }
  while (fscanf(fp, "%s", word) != EOF) {
    find(word, matrixIn, matrixOut);
  }
  fclose(fp);

  /* output result */
  fp = fopen(argv[3], "r");
  if (fp == NULL) {
    printf("Error opening the output file.\n");
    return 1;
  }
  for (i=0; i<DIM; i++) {
    for (j=0; j<DIM; j++) {
      fprintf(fp, "%c", matrixOut[i][j]);
    }
    fprintf(fp, "\n");
  }
  fclose(fp);

  return 0;
} 

/*
 *  find the first char of the word in the matrix
 *  call a function to find all subsequent chars
 */
void find(char word[DIM+1], char matrixIn[DIM][DIM], char matrixOut[DIM][DIM])
{
  int i, j;

  for (i=0; i<DIM; i++) {
    for (j=0; j<DIM; j++) {
      if (matrixIn[i][j]==word[0] && findAll(i, j, word, matrixIn, matrixOut)) {
        return;
      }
    }
  }
}

/*
 *  given the word initial char, find all subsequent chars in the matrix; 
 *  if the word is discovered, save it in the output matrix
 */
int findAll(int row, int col, char word[], char mIn[][DIM], char mOut[][DIM])
{
  char flag;
  int r, c, i, j;
  int offset[2][8] = { {0, -1, -1, -1,  0,  1,  1,  1},
                       {1,  1,  0, -1, -1, -1,  0,  1} };

  for (i=0; i<8; i++) {
    flag = 1;
    for (j=1; j<strlen(word) && flag; j++) {
      r = row + j * offset[0][i];
      c = col + j * offset[1][i];
 
      if (r<0 || r>=DIM || c<0 || c>=DIM || mIn[r][c]!=word[j]) {
        flag = 0;
      }
    }

    if (flag == 1) {
      /* word found */
      for (j=0; j<strlen(word); j++) {
        mOut[row+j*offset[0][i]][col+j*offset[1][i]] = word[j];
      }
      return 1;
    }
  }

  /* word NOT found */
  return 0;
}     

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
