/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio 11 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

#define NAME_TMP1 "nameTmp1.txt"
#define NAME_TMP2 "nameTmp2.txt"
#define DEBUG      1

/* function prototypes */
FILE *fileOpen(char *, char *);
void fileStraightMerge(char *);
void filePrint(char *);

/*
 *  main program
 */
int main(int argc, char *argv[]) 
{
  if (argc != 2) {
    printf("Parameter error.\nRun as: %s <fileName>\n", argv[0]);
  } else {
    fileStraightMerge(argv[1]);
  }
  return EXIT_SUCCESS;
}

/*
 *  perform sorting on file
 */
void fileStraightMerge(char *name) 
{
  FILE *f, *f1, *f2, *fw;
  int n, n1, n2, numData;
  int sf1, sf2, x, y;

  if (DEBUG != 0) {
    printf("Original file:\n");
    filePrint(name);
  }

  n = 1;
  do {
    /* distribution phase: split original file into temporary files */
    f = fileOpen(name, "r");
    f1 = fileOpen(NAME_TMP1, "w");
    f2 = fileOpen(NAME_TMP2, "w");

    fw = f1;
    numData = 0;
    while (fscanf(f, "%d", &x) != EOF) {
      fprintf(fw, "%d\n", x);
      numData++;
      if (numData%n == 0) {
        fw = (fw == f1) ? f2 : f1;
      }
    }
  
    fclose(f);
    fclose(f1);
    fclose(f2);

    if (DEBUG != 0) {
      printf("Distribution phase:\n");
      filePrint(NAME_TMP1);
      filePrint(NAME_TMP2); 
    }

    /* merge phase: merge temporary files into the original file */
    f = fileOpen(name, "w");
    f1 = fileOpen(NAME_TMP1, "r");
    f2 = fileOpen(NAME_TMP2, "r");

    sf1 = fscanf(f1, "%d", &x);
    sf2 = fscanf(f2, "%d", &y);
    n1 = n2 = 0;
    while (sf1!=EOF && sf2!=EOF) {
      if (x < y) { 
        fprintf(f, "%d ", x);
        n1++;
        sf1 = fscanf(f1, "%d", &x);
        if (sf1==EOF || n1>=n) {
          do {
            fprintf(f, "%d ", y);
            sf2 = fscanf(f2, "%d", &y);
            n2++;
          } while (sf2!=EOF && n2<n);
          n1 = n2 = 0;
        }
      } else {
        fprintf(f, "%d ", y);
        n2++;
        sf2 = fscanf(f2, "%d", &y);
        if (sf2==EOF || n2>=n) {
          do {
            fprintf(f, "%d ", x);
            sf1 = fscanf(f1, "%d", &x);
            n1++;
          } while (sf1!=EOF && n1<n);
          n1 = n2 = 0;
        }
      }
    }

    /* copy tail of file 1 */
    while (sf1 != EOF) {
      fprintf(f, "%d ", x);
      sf1 = fscanf(f1, "%d", &x);
    }

    /* copy tail of file 2 */
    while (sf2 != EOF) {
      fprintf(f, "%d ", y);
      sf2 = fscanf(f2, "%d", &y);
    }

    fclose(f);
    fclose(f1);
    fclose(f2);

    if (DEBUG != 0) {
      printf("Merge phase:\n");
      filePrint(name);
    }

    n = n*2;
  } while (n < numData);
}

/*
 *  print the contents of a given file
 */
void filePrint(char *name) 
{
  FILE *fp;
  int n;

  fp = fileOpen(name, "r");
  while (fscanf(fp, "%d", &n) != EOF) {
    printf("%d ", n);
  }
  printf("\n");

  fclose(fp);
}

/*
 *  open a file, checking the correctness of the operation
 */
FILE *fileOpen(char *name, char *access) 
{
  FILE *fp;

  fp = fopen(name, access);
  if (fp == NULL) {
    printf("Error opening file %s.\n", name);
    exit(EXIT_FAILURE);
  }
  return fp;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
