/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio Proposto 05 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define  MAX_DIM   200
#define  MAX_LINE  1000
#define  MAP_NAME  "map.txt"

/* function prototypes */
int map_read(int map[MAX_DIM][MAX_DIM], int *row_ptr, int *col_ptr);
void map_search(int map[MAX_DIM][MAX_DIM], int nrow, int ncol);

/*
 *  main program
 */
int main(void)
{
  int map[MAX_DIM][MAX_DIM], n, m;

  if (map_read(map, &n, &m)) {
    map_search(map, n, m);
  }
  return 0;
}

/*
 *  read the map from file
 */
int map_read(int map[MAX_DIM][MAX_DIM], int *row_ptr, int *col_ptr)
{
  char line[MAX_LINE], *ptr;
  int i, j, val, done;
  FILE *fp;

  /* file input */
  fp = fopen(MAP_NAME, "r");
  if (fp == NULL) {
    printf("Input file not readable!\n");
    return 0;
  }
  for (i=0; fgets(line, MAX_LINE, fp)!=NULL; i++) {
    done = j = 0;
    ptr = line;
    do { 
      if (sscanf(ptr, "%d", &val) == 1) {
        map[i][j++] = val;
        while (isspace(*ptr)) {
          ptr++;
        }
        while (isdigit(*ptr)) {
          ptr++;
        }
      } else {
        done = 1;
      }
    } while (!done);
  }
  fclose(fp);

  *row_ptr = i;
  *col_ptr = j;
  return 1;
}

/*
 *  scan the map looking for geographic elements
 */
void map_search(int map[MAX_DIM][MAX_DIM], int nrow, int ncol)
{
  int i, j, h;

  for (i=1; i<nrow-1; i++) {
    for (j=1; j<ncol-1; j++) {
      h = map[i][j];
      if (h>map[i-1][j] && h>map[i+1][j] && h>map[i][j-1] && h>map[i][j+1]) {
        printf("Peak: [%d][%d], altitude=%d\n", i, j, h);
      } else if (h<map[i-1][j] && h<map[i+1][j] && h<map[i][j-1] && h<map[i][j+1]) {
        printf("Hollow: [%d][%d], altitude=%d\n", i, j, h);
      } else if (h<map[i-1][j] && h<map[i+1][j] && h>map[i][j-1] && h>map[i][j+1]) {
        printf("Pitch: [%d][%d], altitude=%d\n", i, j, h);
      } else if (h>map[i-1][j] && h>map[i+1][j] && h<map[i][j-1] && h<map[i][j+1]) {
        printf("Pitch: [%d][%d], altitude=%d\n", i, j, h);
      }
    }
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
