/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio Proposto 06 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

/* constants */
#define  NUM       10
#define  LENGTH    20
#define  MINE      '*'
#define  EMPTY     ' '
#define  UNKNOWN   ' '

/* function prototypes */
int read_board(char board[NUM+2][NUM+2]);
int init_display(char board[NUM+2][NUM+2], char display[NUM+2][NUM+2]);
void print_board(char bd[NUM+2][NUM+2]);
int get_move(char display[NUM+2][NUM+2]);
int update_display(char in[NUM+2][NUM+2], char out[NUM+2][NUM+2], int move);

/*
 *  main program
 */
int main(void)
{
  char board[NUM+2][NUM+2], display[NUM+2][NUM+2];
  int num_mines, num_to_guess, move, dead;

  if (read_board(board)) {
    return 1;
  }
  num_mines = init_display(board, display);
  num_to_guess = NUM*NUM - num_mines;

  do {
    print_board(display);
    move = get_move(display);
    dead = update_display(board, display, move);
    num_to_guess--;
  } while (!dead && num_to_guess>0);

  print_board(board);
  if (dead) {
    printf("You are DEAD.\n");
  } else {
    printf("Cogratulations, game solved.\n");
  }
  return 0;
}

/*
 *  read the game board from file; return 1 upon errors
 */
int read_board(char board[NUM+2][NUM+2])
{
  char name[LENGTH], line[LENGTH];
  FILE *fp;
  int i, j;

  printf("Input the file name: ");
  scanf("%s%*c", name);
  fp = fopen(name, "r");
  if (fp == NULL) {
    printf("Error opening the input file.\n");
    return 1;
  }

  for (j=0; j<=NUM+1; j++) {
    board[0][j] = board[NUM+1][j] = EMPTY;
  }

  for (i=1; i<=NUM; i++) {
    if ((fgets(line, LENGTH, fp)==NULL) || (strlen(line)!=NUM+1)) {
      printf("Error parsing the input file.\n");
      fclose(fp);
      return 1;
    }
    board[i][0] = EMPTY;
    for (j=0; j<NUM; j++) {
      board[i][j+1] = line[j];
    }
    board[i][NUM+1] = EMPTY;
  }

  fclose(fp);
  return 0;
}

/*
 *  init the game boards with the proper signs
 */
int init_display(char board[NUM+2][NUM+2], char display[NUM+2][NUM+2])
{
  int i, j, k, num_mines=0, num_adj;
  int xoff[8] = {0, 1, 1, 1, 0, -1, -1, -1};
  int yoff[8] = {-1, -1, 0, 1, 1, 1, 0, -1};

  for (i=1; i<=NUM; i++) {
    for (j=1; j<=NUM; j++) {
      display[i][j] = UNKNOWN;
      if (board[i][j] != MINE) {
        num_adj = 0;
        for (k=0; k<8; k++) {
          if (board[i+xoff[k]][j+yoff[k]] == MINE) {
            num_adj++;
          }
        }
        board[i][j] = num_adj + '0';
      } else {
        num_mines++;
      }
    }
  }
  return num_mines;
}

/*
 *  print the game board
 */
void print_board(char bd[NUM+2][NUM+2])
{
  int i, j;

  printf("   ");
  for (i=0; i<NUM; i++) {
    printf("%d", i);
  }
  printf("\n  +");
  for (i=0; i<NUM; i++) {
    printf("-");
  }
  printf("+\n");

  for (i=1; i<=NUM; i++) {
    printf("%c.|", 'a'+i-1);
    for (j=1; j<=NUM; j++) {
      printf("%c", bd[i][j]);
    }
    printf("|.%c\n", 'a'+i-1);
  }

  printf("  +");
  for (i=0; i<NUM; i++) {
    printf("-");
  }
  printf("+\n   ");
  for (i=0; i<NUM; i++) {
    printf("%d", i);
  }
  printf("\n\n");
}

/*
 *  parse a player move
 */
int get_move(char display[NUM+2][NUM+2])
{
  int move_int, row, col, valid=0;
  char move_string[LENGTH];

  while (!valid) {
    printf("Make your move: ");
    fgets(move_string, LENGTH, stdin);
    if (strlen(move_string) != 3) {
      while (move_string[strlen(move_string)-1] != '\n') {
        fgets(move_string, LENGTH, stdin);
      }
      printf("Not a valid move!\n");
      continue;
    }
    row = (toupper(move_string[0]) - 'A');
    if ((row < 0) || (row > 9)) {
      printf("Not a valid move!\n");
      continue;
    }
    col = move_string[1] - '0';
    if ((col < 0) || (col > 9)) {
      printf("Not a valid move!\n");
      continue;
    }
    if (display[row+1][col+1] != UNKNOWN) {
      printf("Not a valid move!\n");
      continue;
    }
    valid = 1;
    move_int = row*NUM + col;
  }
  return move_int;
}

/*
 *  update the display board; return 1 upon a mine
 */
int update_display(char in[NUM+2][NUM+2], char out[NUM+2][NUM+2], int move)
{
  int row, col;

  row = (move / NUM) + 1;
  col = (move % NUM) + 1;
  out[row][col] = in[row][col];

  return (in[row][col] == MINE);
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
