/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 10 - Esercizio Proposto 07 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

/* constants */
#define  NUM       10
#define  LENGTH    20
#define  SHIP      '*'
#define  EMPTY     ' '
#define  UNKNOWN   ' '
#define  WATER     'X'
#define  STRIKEN   'O'

/* function prototypes */
int read_board(char board[4][NUM][NUM], int which);
int check_board(char brd[NUM][NUM]);
void print_board(char board[4][NUM][NUM], int offset);
void print_final(char board[4][NUM][NUM]);
int get_move(int player, char brd[NUM][NUM]);
int update_board(char board[4][NUM][NUM], int move, int player);

/*
 *  main program
 */
int main(void)
{
  char board[4][NUM][NUM];
  int player=1, win, move;

  if (read_board(board, 0) || read_board(board, 1)) {
    return 1;
  }
  printf("\n");

  do {
    player = 1-player;
    print_board(board, 2);
    move = get_move(player, board[3-player]);
    win = update_board(board, move, player);
  } while (!win);

  printf("Player %d won!\n\n", player+1);
  print_final(board);
  return 0;
}

/*
 *  read a game board from file; return 1 upon errors
 */
int read_board(char board[4][NUM][NUM], int which)
{
  char name[LENGTH], line[LENGTH];
  FILE *fp;
  int i, j;

  printf("Input player %d file name: ", which+1);
  scanf("%s%*c", name);
  fp = fopen(name, "r");
  if (fp == NULL) {
    printf("Error opening the input file.\n");
    return 1;
  }

  for (i=0; i<NUM; i++) {
    if ((fgets(line, LENGTH, fp)==NULL) || (strlen(line)!=NUM+1)) {
      printf("Error parsing the input file.\n");
      fclose(fp);
      return 1;
    }
    for (j=0; j<NUM; j++) {
      board[which][i][j] = line[j];
      board[which+2][i][j] = UNKNOWN;
    }
  }

  fclose(fp);
  return check_board(board[which]);
}

/*
 *  check the correctness of a game board; return 1 upon errors
 */
int check_board(char brd[NUM][NUM])
{
  int i, j, k, dim, valid, num[4] = {4, 3, 2, 1};

  /* invalid chars */
  for (i=0; i<NUM; i++) {
    for (j=0; j<NUM; j++) {
      if (brd[i][j]!=SHIP && brd[i][j]!=EMPTY) {
        printf("Invalid symbol found: aborting.\n");
        return 1;
      }
    }
  }

  /* invalid shapes */
  valid = 1;
  for (i=0; i<NUM && valid; i++) {
    for (j=0; j<NUM && valid; j++) {
      if (brd[i][j] == SHIP) {
        if ((i>0 && j>0 && brd[i-1][j-1]==SHIP) ||
            (i>0 && j<NUM-1 && brd[i-1][j+1]==SHIP) ||
            (i<NUM-1 && j<NUM-1 && brd[i+1][j+1]==SHIP) ||
            (i<NUM-1 && j>0 && brd[i+1][j-1]==SHIP)) {
          valid = 0;
        }
      }
    }
  }
  if (!valid) {
    printf("Invalid shape detected: aborting.\n");
    return 1;
  }

  /* invalid ships number and dimension */
  for (i=0; i<NUM; i++) {
    for (j=0; j<NUM; j++) {
      if (brd[i][j] == SHIP) {
        dim = 1;
        brd[i][j] = STRIKEN;

        /* check for a horizontal ship */
        k = j;
        while (k<NUM-1 && brd[i][++k]==SHIP) {
          brd[i][k] = STRIKEN;
          dim++;
        }

        /* check for a vertical ship */
        k = i;
        while (k<NUM-1 && brd[++k][j]==SHIP) {
          brd[k][j] = STRIKEN;
          dim++;
        }

        if (dim > 4) {
          printf("Invalid ship dimension: aborting.\n");
          return 1;
        }
        num[dim-1]--;
      }
    }
  }
  if (num[0] || num[1] || num[2] || num[3]) {
    printf("Invalid ship number found: aborting.\n");
    return 1;
  }

  /* restore the SHIP symbols */
  for (i=0; i<NUM; i++) {
    for (j=0; j<NUM; j++) {
      if (brd[i][j] == STRIKEN) {
        brd[i][j] = SHIP;
      }
    }
  }

  return 0;
}

/*
 *  print the game board
 */
void print_board(char board[4][NUM][NUM], int offset)
{
  int i, j;

  printf("      P1 NAVY    |      P2 NAVY\n");
  printf("   P2 fires here |   P1 fires here\n");
  for (i=0; i<NUM; i++) {
    printf("  %c. ", 'a'+i);
    for (j=0; j<NUM; j++) {
      printf("%c", board[offset][i][j]);
    }
    printf("  |  %c. ", 'a'+i);
    for (j=0; j<NUM; j++) {
      printf("%c", board[offset+1][i][j]);
    }
    printf("\n");
  }
  printf("     ");
  for (i=0; i<NUM; i++) {
     printf("%d", i);
  }
  printf("  |     ");
  for (i=0; i<NUM; i++) {
     printf("%d", i);
  }
  printf("\n");
}

/*
 *  print the (final) game board, indicating the SHIP positions
 */
void print_final(char board[4][NUM][NUM])
{
  int i, j, k;

  for (k=0; k<2; k++) {
    for (i=0; i<NUM; i++) {
      for (j=0; j<NUM; j++) {
        if (board[i][j][k] == SHIP) {
          board[i][j][k+2] = SHIP;
        }
      }
    }
  }
  print_board(board, 2);
}

/*
 *  parse a player move
 */
int get_move(int player, char brd[NUM][NUM])
{
  int move_int, row, col, valid=0;
  char move_string[LENGTH];

  while (!valid) {
    printf("Player %d, fire: ", player+1);
    fgets(move_string, LENGTH, stdin);
    if (strlen(move_string) != 3) {
      while (move_string[strlen(move_string)-1] != '\n') {
        fgets(move_string, LENGTH, stdin);
      }
      printf("Not a valid move.\n");
      continue;
    }
    row = (toupper(move_string[0]) - 'A');
    if ((row < 0) || (row > 9)) {
      printf("Not a valid move.\n");
      continue;
    }
    col = move_string[1] - '0';
    if ((col < 0) || (col > 9)) {
      printf("Not a valid move.\n");
      continue;
    }
    if (brd[row][col] != UNKNOWN) {
      printf("Not a valid move.\n");
      continue;
    }
    valid = 1;
    move_int = row*NUM + col;
  }
  return move_int;
}

/*
 *  update a player board; return 1 upon the victory
 */
int update_board(char board[4][NUM][NUM], int move, int mp)
{
  int i, j, k, l, hv = -1;

  i = (move / NUM);
  j = (move % NUM);
  k = 1-mp;

  if (board[k][i][j] != SHIP) {
    board[k+2][i][j] = WATER;
    printf("Missed.\n\n");
    return 0;
  }

  board[k+2][i][j] = board[k][i][j] = STRIKEN;
  if ((i>0 && board[k][i-1][j]!=EMPTY) || (i<NUM-1 && board[k][i+1][j]!=EMPTY)) {
    hv = 1;
  }
  if ((j>0 && board[k][i][j-1]!=EMPTY) || (j<NUM-1 && board[k][i][j+1]!=EMPTY)) {
    hv = 0;
  }

  if (hv == 1) {
    /* vertical ship */
    l = i;
    while (--l>=0 && board[k][l][j]!=EMPTY) {
      if (board[k][l][j] == SHIP) {
        printf("Hit.\n\n");
        return 0;
      }
    }
    l = i;
    while (++l<NUM && board[k][l][j]!=EMPTY) {
      if (board[k][l][j] == SHIP) {
        printf("Hit.\n\n");
        return 0;
      }
    }
  }

  if (hv == 0) {
    /* horizontal ship */
    l = j;
    while (--l>=0 && board[k][i][l]!=EMPTY) {
      if (board[k][i][l] == SHIP) {
        printf("Hit.\n\n");
        return 0;
      }
    }
    l = j;
    while (++l<NUM && board[k][i][l]!=EMPTY) {
      if (board[k][i][l] == SHIP) {
        printf("Hit.\n\n");
        return 0;
      }
    }
  }
  printf("Sunk.\n\n");

  /* check for victory */
  for (i=0; i<NUM; i++) {
    for (j=0; j<NUM; j++) {
      if (board[k][i][j] == SHIP) {
        return 0;
      }
    }
  }
  return 1;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
