/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio 05 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>

#define MAX_ARRAY  50
#define MAX_NAME   21

/* structure declaration */
typedef struct {
  char name[MAX_NAME];
  int id, mark;
} student_t;

/* function prototypes */
int compare(student_t st1, student_t st2, int key);
void swap(student_t array[MAX_ARRAY], int i, int j);

/*
 *  students comparison according to key
 */
int compare(student_t st1, student_t st2, int key)
{
  switch (key) {
    case 1:  return strcmp(st1.name, st2.name);
    case 2:  return st1.id - st2.id;
    default: return st1.mark - st2.mark;
  }
}

/*
 *  swap the contents of two students
 */
void swap(student_t array[MAX_ARRAY], int i, int j) 
{
  student_t temp;

  temp = array[i];
  array[i] = array[j];
  array[j] = temp;
}

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  int n, i, j, key, id, mark, totalMark, posMin;
  student_t array[MAX_ARRAY];
  char name[MAX_NAME];
  FILE *fp;

  if (argc < 2) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <input_file>.\n", argv[0]);
    return 1;
  }

  /* load file */
  fp = fopen(argv[1], "r");
  if (fp == NULL) {
    printf("Error: input file not readable.\n");
    return 1;
  }

  n = totalMark = 0;
  while (n<MAX_ARRAY && fscanf(fp, "%s%d%d", name, &id, &mark)!=EOF) {
    strcpy(array[n].name, name);
    array[n].id = id;
    array[n].mark = mark;
    totalMark += mark;
    n++;
  }
  fclose(fp);

  /* selection sort */
  printf("Key for sorting? [1=name, 2=id, 3=mark] ");
  scanf("%d", &key);
  for (i=0; i<n-1; i++) {
    posMin = i;
    for (j=i+1; j<n; j++) {
      if (compare(array[j], array[posMin], key) < 0) {
        posMin = j;
      }
    }
    swap(array, i, posMin);
  }

  /* display results */
  printf("Sorted database:\n");
  for (i=0; i<n; i++) {
    printf("%20s %5d %3d\n", array[i].name, array[i].id, array[i].mark);
  }
  printf("\nAverage mark: %.1f\n", ((float)totalMark)/n);

  return 0;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
