/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio 06 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAX_POINT  100

/* structure declaration */
struct point {
  float x, y, z;  
  float distance;
};

/* function prototypes */
int read_points(char *, struct point []);
void sort_points(struct point [], int);
void write_points(char *, struct point [], int);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  struct point space_points[MAX_POINT];
  int n_points;

  if (argc < 3) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <input_file> <output_file>.\n", argv[0]);
    return EXIT_FAILURE;
  }

  n_points = read_points(argv[1], space_points);
  sort_points(space_points, n_points);
  write_points(argv[2], space_points, n_points);

  return EXIT_SUCCESS;
}

/*
 *  read the points input file
 */
int read_points(char *name, struct point points[])
{
  float x, y, z;
  FILE *fp;
  int i=0;

  if ((fp=fopen(name, "r")) == NULL) {
    printf("Error opening file %s.\n", name);
    exit(EXIT_FAILURE);
  }
  while (fscanf(fp, "%f%f%f", &x, &y, &z)!=EOF && i<MAX_POINT) {
    points[i].x = x;
    points[i].y = y;
    points[i].z = z;
    points[i].distance = sqrt(pow(x, 2)+pow(y, 2)+pow(z, 2));
    i++;
  }
  fclose(fp);

  return i;
}

/*
 *  sort the points (on distance)
 */
void sort_points(struct point points[], int n_points)
{
  int i, j;
  struct point current;

  for (i=1; i<n_points; i++) {
    current = points[i];
    j = i;
    while ((--j >= 0) && (current.distance < points[j].distance)) {
      points[j+1] = points[j];
    }
    points[j+1] = current;
  }
}

/*
 *  write the points output file
 */
void write_points(char *name, struct point points[], int n_points)
{
  FILE *fp;
  int i;

  if ((fp=fopen(name, "w")) == NULL) {
    printf("Error opening file %s.\n", name);
    exit(EXIT_FAILURE);
  }
  for (i=0; i<n_points; i++) {
    fprintf(fp, "%f %f ", points[i].x, points[i].y);
    fprintf(fp, "%f %f\n", points[i].z, points[i].distance);
  }
  fclose(fp);
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
