/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio 08 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAX_NAME    20
#define MAX_DATE    10
#define MAX_LINE    100
#define MAX_PERSON  1000

/* structure declaration */
typedef struct {
  char name[MAX_NAME+1];
  char birth[MAX_DATE+1];
  char death[MAX_DATE+1];
} person_t;

/* function prototypes */
int read_births(person_t []);
int read_deaths(person_t [], int );
int write_output(person_t [], int );
int find_person(person_t [], int, char *);

/*
 *  main program
 */
int main(void) 
{
  int num_person;
  person_t people[MAX_PERSON];

  num_person = read_births(people);
  if (num_person == 0) {
    return 1;
  }
  if (read_deaths(people, num_person) == 0) {
    return 1;
  }
  return write_output(people, num_person);
}

/*
 *  births (and names) storage
 */
int read_births(person_t people[])
{
  char line[MAX_LINE];
  FILE *fp;
  int i=0;

  fp = fopen("births.txt", "r");
  if (fp == NULL) {
    printf("Error opening the birth file.\n");
    return 0;
  }
  while (fgets(line, MAX_LINE, fp)!=NULL && i<MAX_PERSON) {
    while (isspace(line[strlen(line)-1])) {
      line[strlen(line)-1] = '\0';
    }
    strcpy(people[i].name, line);

    fgets(line, MAX_LINE, fp);
    while (isspace(line[strlen(line)-1])) {
      line[strlen(line)-1] = '\0';
    }
    strcpy(people[i].birth, line);

    i++;
  }
  fclose(fp);

  return i;
}

/*
 *  deaths storage
 */
int read_deaths(person_t people[], int num_person)
{
  char line[MAX_LINE];
  FILE *fp;
  int i;

  fp = fopen("deaths.txt", "r");
  if (fp == NULL) {
    printf("Error opening the death file.\n");
    return 0;
  }
  while (fgets(line, MAX_LINE, fp) != NULL) {
    while (isspace(line[strlen(line)-1])) {
      line[strlen(line)-1] = '\0';
    }
    i = find_person(people, num_person, line);
    if (i < 0) {
      fclose(fp);
      printf("Error: incompatible files.\n");
      return 0;
    }

    fgets(line, MAX_LINE, fp);
    while (isspace(line[strlen(line)-1])) {
      line[strlen(line)-1] = '\0';
    }
    strcpy(people[i].death, line);
  }
  fclose(fp);

  return 1;
}

/*
 *  output result
 */
int write_output(person_t people[], int num_person)
{
  int i;
  FILE *fp;

  fp = fopen("lives.txt", "w");
  if (fp == NULL) {
    printf("Error opening the output file.\n");
    return 1;
  }
  for (i=0; i<num_person; i++) {
    fprintf(fp, "%-20s  ", people[i].name);
    fprintf(fp, "%+10s  ", people[i].birth); 
    fprintf(fp, "%+10s\n", people[i].death);
  }
  fclose(fp);
  return 0;
}

/*
 *  find the position of a person in an array (given the name)
 */
int find_person(person_t people[], int num_person, char *name)
{
  int i;

  for (i=0; i<num_person; i++) {
    if (strcmp(people[i].name, name) == 0) {
      return i;
    }
  }
  return -1;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
