/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio 09 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>

#define MAX     100
#define LENGTH   30

/* structure declaration */
typedef struct {
  char value;
  int visited;
} element_t;

/* function prototypes */
int mapRead(element_t [MAX][MAX], int *, int *);
int lengthEvaluate(element_t [MAX][MAX], int, int, int, int);

/*
 *  main program
 */
int main(void)
{
  element_t map[MAX][MAX];
  int nSnake=0, minLen=MAX+1, maxLen=0;
  int nr, nc, i, j, length;

  /* map load */
  if (!mapRead(map, &nr, &nc)) {
    return 1;
  }

  /* computation */
  for (i=0; i<nr; i++) {
    for (j=0; j<nc; j++) {
      if (map[i][j].value == '+') {
        /* found a new snake: evaluate the length */
        nSnake++; 
        length = lengthEvaluate(map, nr, nc, i, j);
        if (length < minLen) {
          minLen = length;
        } 
        if (length > maxLen) {
          maxLen = length;
        }
      }
    }
  }

  printf("Snake number = %d\n", nSnake);
  printf("Min length = %d\n", minLen);
  printf("Max length = %d\n", maxLen);
  return 0;
}  

/*
 *  load the map from file
 */
int mapRead(element_t map[MAX][MAX], int *nr, int *nc)
{
  char fname[LENGTH], c;
  FILE *fp;
  int i, j;

  /* open file */
  printf("Input file name: ");
  scanf("%s", fname);
  fp = fopen(fname, "r");
  if (fp == NULL) {
    printf("Error: input file not readable.\n");
    return 0;
  }

  /* parse file (with check) */
  if (fscanf(fp, "%d %d", nr, nc) != 2) {
    printf("Error while parsing the input file.\n");
    fclose(fp);
    return 0;
  }
  fscanf(fp, "%c", &c); /* skip the '\n'*/

  for (i=0; i<*nr; i++) {
    for (j=0; j<*nc; j++) {
      if (fscanf(fp, "%c", &map[i][j].value) != EOF) {
        map[i][j].visited = 0; 
      } else {
        printf("Error while parsing the input file.\n");
        fclose(fp);
        return 0;
      }
    }
    fscanf(fp, "%c", &c); /* skip the '\n'*/
  }

  fclose(fp);
  return 1;
}

/*
 *  evaluate the length of a snake, given its "head" position
 */
int lengthEvaluate(element_t map[MAX][MAX], int nr, int nc, int r, int c)
{
  int end=0, length=1, found, r_new, c_new, i, j;

  /* set the visited field for the snake "head" */
  map[r][c].visited = 1;

  while (!end) {
    /* check the contiguous positions */
    found = 0;  
    for (i=r-1; i<=r+1; i++) {
      for (j=c-1; j<=c+1; j++) {
        if (i>=0 && i<nr && j>=0 && j<nc) {
          if (map[i][j].value=='*' && !map[i][j].visited) {
            /* found a new snake portion */          
            found = 1;
            r_new = i;
            c_new = j;
          }
        }
      }
    }

    if (found) {
      length++;
      r = r_new;
      c = c_new;
      map[r][c].visited = 1;
    } else {
      end = 1;
    }
  }

  return length;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
