/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio 10 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>

/* structure declaration */
typedef struct {
  char func;
  int  callNum;
  char calls[26];
} func_t;

/* function prototypes */
int load(func_t array[], char *name);
void compute(func_t array[], int n);
int search(func_t array[], int n, char f);
void append(func_t array[], int i, char f);
void print(func_t array[], int n);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  func_t array[26];
  int funcNum;

  if (argc < 2) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <input_file>.\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  funcNum = load(array, argv[1]);
  compute(array, funcNum);
  print(array, funcNum);
  return EXIT_SUCCESS;
}

/*
 *  load the input file contents
 */
int load(func_t array[], char *name)
{
  int i, n=0;
  FILE *fin;

  fin = fopen(name, "r");
  if (fin == NULL) {
    printf("Error opening the input file.\n");
    exit(EXIT_FAILURE);
  }

  /* parse the file */
  while (fscanf(fin, "%c %d", &array[n].func, &array[n].callNum) != EOF) {
    for (i=0; i<array[n].callNum; i++) {
      fscanf(fin, " %c", &array[n].calls[i]);
    }
    fscanf(fin, "%*c");
    n++;
  }

  fclose(fin);
  return n;
}

/*
 * compute the function calls map
 */
void compute(func_t array[], int n)
{
  int i, j, k, l;

  /* for each function f[i] */
  for (i=0; i<n; i++) {
    /* for each function called by f[i] */
    for (j=0; j<array[i].callNum; j++) {
      /* search this function as a calling one */
      k = search(array, n, array[i].calls[j]);
      /* now f[k] (k>=0) is a called function */
      /* append to f[i] all functions called by f[k] */
      if (k >= 0) {
        for (l=0; l<array[k].callNum; l++) {
          append(array, i, array[k].calls[l]);
        }
      }
    }
  }
}

/*
 *  search a function f in the function array 
 */
int search(func_t array[], int n, char f)
{
  int k;

  for (k=0; k<n; k++) {
    if (array[k].func == f) {
      return k;
    }
  }

  return -1;
}

/*
 *  append a funtion f to the set of functions called by array[i]
 */
void append(func_t array[], int i, char f)
{
  int k;

  /* search f in array[i].calls */
  for (k=0; k<array[i].callNum; k++) {
    if (array[i].calls[k] == f) {
      /* called function already there: nothing to do */
      return;
    }
  }

  /* not found, append f to array[i].calls */
  array[i].calls[k] = f;
  array[i].callNum++;
}

/*
 *  output results
 */
void print(func_t array[], int n)
{
  int i, j;

  printf("Function calls map:\n");
  for (i=0; i<n; i++) {
    printf("%c calls", array[i].func);
    for (j=0; j<array[i].callNum; j++) {
      printf(" %c", array[i].calls[j]);
    }
    printf("\n");
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
