/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio Proposto 02 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <math.h>

/* structure declaration */
typedef struct {
  float x;
  float y;
} point_t;

/* function prototypes */
int read_point(FILE *fp, point_t *pt);
float line_length(point_t p1, point_t p2);
float line_area(point_t p1, point_t p2);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  point_t old, new;
  float area, length;
  FILE *fp;

  if (argc < 2) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <input_file>.\n", argv[0]);
    return 1;
  }

  fp = fopen(argv[1], "r");
  if (fp == NULL) {
    printf("Error: input file not readable.\n");
    return 1;
  }

  area = length = 0;
  read_point(fp, &old);
  while (read_point(fp, &new)) {
    length += line_length(old, new);
    area += line_area(old, new);
    old = new;
  }
  fclose(fp);

  printf("Line length = %.4f\n", length);
  printf("Line area   = %.4f\n", area);
  return 0;
}

/*
 *  point input
 */
int read_point(FILE *fp, point_t *pt)
{
  if (fscanf(fp, "%f%f", &pt->x, &pt->y) != 2) {
    return 0;
  }
  return 1;
}

/*
 *  distance computation
 */
float line_length(point_t p1, point_t p2)
{
  float xd, yd;

  xd = p2.x - p1.x;
  yd = p2.y - p1.y;

  return sqrt(pow(xd, 2) + pow(yd, 2));
}

/*
 *  area computation
 */
float line_area(point_t p1, point_t p2)
{
  float xd, yp;

  xd = p2.x - p1.x;
  yp = p2.y + p1.y;

  return xd * yp /2;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
