/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio Proposto 03 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define NAME_LENGTH  31
#define ID_LENGTH     6
#define N_STUDENT   100

/* structure declaration */
struct student {
  char name[NAME_LENGTH];
  char id[ID_LENGTH];
  int mark;
};

/* function prototypes */
int readStudents(struct student [], char *);
void readMarks(struct student [], int, char *);
void writeResults(struct student [], int, char *);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  struct student array[N_STUDENT];
  int num;

  if (argc < 4) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <student_file> <mark_file> <output_file>.\n", argv[0]);
    return EXIT_FAILURE;
  }

  num = readStudents(array, argv[1]);
  readMarks(array, num, argv[2]);
  writeResults(array, num, argv[3]);

  return EXIT_SUCCESS;
}

/*
 *  load the student file, return the student number
 */
int readStudents(struct student array[], char *fileName)
{
  FILE *fp;
  int i;

  fp = fopen(fileName, "r");
  if (fp == NULL) {
    printf("Error opening file %s.\n", fileName);
    exit(EXIT_FAILURE);
  }

  /* parse file */
  i = 0;
  while (i<N_STUDENT && fscanf(fp, "%s %s", array[i].name, array[i].id)!=EOF) {
    i++;
  }
  fclose(fp);
 
  return i;
}

/*
 *  load the mark file, compute the final mark
 */
void readMarks(struct student array[], int num, char *fileName)
{
  char id[ID_LENGTH];
  int i, m1, m2, found;
  float mean;
  FILE *fp;
  
  fp = fopen(fileName, "r");
  if (fp == NULL) {
    printf("Error opening file %s.\n", fileName);
    exit(EXIT_FAILURE);
  }

  /* parse file */
  while (fscanf(fp, "%s %d %d", id, &m1, &m2) != EOF) {
    /* search the student id in the array */
    found = 0;
    for (i=0; i<num && !found; i++) {
      if (strcmp(id, array[i].id) == 0) {
        /* id found */
        found = 1;
        if (m1==30 && m2==30) {
          array[i].mark = 31;
        } else if (m1<15 || m2<15) {
          array[i].mark = 15;
        } else {
          mean = (m1+m2) / 2.0;
          if (mean < 18) {
            array[i].mark = 15;
          } else {
            array[i].mark = ceil(mean);
          }
        }
      }
    }
    if (!found) {
      printf("Student id NOT found (%s): aborting.\n", id);
      exit(EXIT_FAILURE);
    }
  }
  fclose(fp);
}

/*
 *  write the output file
 */
void writeResults(struct student array[], int num, char *fileName)
{
  FILE *fp;
  int i;

  fp = fopen(fileName, "w");
  if (fp == NULL) {
    printf("Error opening file %s.\n", fileName);
    exit(EXIT_FAILURE);
  }

  fprintf(fp, "%5s %30s %s\n", "ID", "Name", "Mark");
  for (i=0; i<num; i++) {
    fprintf(fp, "%s %30s ", array[i].id, array[i].name);

    if (array[i].mark < 18) {
      fprintf(fp, "failed\n");
    } else if (array[i].mark == 31) {
      fprintf(fp, "30 cum laude\n");
    } else {
      fprintf(fp, "%d\n", array[i].mark);
    }
  }
  fclose(fp);
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
