/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio Proposto 04 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <math.h>

#define NP 10

/* structure declaration */
typedef struct {
  float x;
  float y;
  float z;
  int analyzed;
} point_t;

/* function prototypes */
int read_points(point_t []);
int choose_point(point_t [], point_t);
float distance(point_t, point_t);

/*
 *  main program
 */
int main(void) 
{
  point_t pts[NP], origin={0, 0, 0, -1};
  int i, best;

  if (read_points(pts)) {
    printf("Best path is:\n");
    for (i=0; i<NP; i++) {
      best = choose_point(pts, (i==0) ? origin : pts[best]);
      pts[best].analyzed = i;
      printf("P[%d] = (%+.2f, ", i, pts[best].x);
      printf("%+.2f, %+.2f)\n", pts[best].y, pts[best].z);
    }
  }
  return 0;
}

/*
 *  points storage
 */
int read_points(point_t pts[NP])
{
  char name[20];
  FILE *fin;
  int i;

  printf("Input file name: ");
  scanf("%s", name);
  fin = fopen(name, "r");
  if (fin == NULL) {
    printf("Error: input file not readable.\n");
    return 0;
  }

  for (i=0; i<NP; i++) {
    if (fscanf(fin, "%f%f%f", &pts[i].x, &pts[i].y, &pts[i].z) != 3) {
      printf("Error while parsing the input file.\n");
      fclose(fin);
      return 0;
    }
    pts[i].analyzed = -1;
  }
  fclose(fin);
  return 1;
}

/*
 *  point selection
 */
int choose_point(point_t pts[NP], point_t start)
{
  float dist, min_dist=-1;
  int i, best;

  for (i=0; i<NP; i++) {
    if (pts[i].analyzed < 0) {
      dist = distance(start, pts[i]);
      if ((dist < min_dist) || (min_dist < 0)) {
        min_dist = dist;
        best = i;
      }
    }
  }

  return best;
}

/*
 *  distance computation
 */
float distance(point_t p1, point_t p2)
{
  float xd, yd, zd;

  xd = p2.x - p1.x;
  yd = p2.y - p1.y;
  zd = p2.z - p1.z;

  return sqrt(pow(xd, 2) + pow(yd, 2) + pow(zd, 2));
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
