/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio Proposto 05 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define MAX_NAME    20
#define MAX_LINE    100
#define MAX_PERSON  1000

const int month_days[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

/* structure declaration */
typedef struct {
  int day;
  int month;
  int year;
} date_t;

typedef struct {
  char name[MAX_NAME+1];
  date_t birth;
  date_t death;
  int life_days;
} person_t;

/* function prototypes */
int read_input(person_t []);
void sort_array(person_t [], int);
void write_output(person_t [], int);
int date_difference(date_t, date_t);

/*
 *  main program
 */
int main(void) 
{
  int num_person;
  person_t people[MAX_PERSON];

  num_person = read_input(people);
  sort_array(people, num_person);
  write_output(people, num_person);

  return EXIT_SUCCESS;
}

/*
 *  read in the input file 
 */
int read_input(person_t people[])
{
  char line[MAX_LINE], *aux;
  FILE *fp;
  int i=0;

  fp = fopen("lifes.txt", "r");
  if (fp == NULL) {
    printf("Error opening the input file.\n");
    exit(EXIT_FAILURE);
  }

  while (fgets(line, MAX_LINE, fp) != NULL) {
    aux = strchr(line, '/') - 1;
    while (isdigit(*aux) || isspace(*aux)) {
      aux--;
    }
    *(aux+1) = '\0';
    strcpy(people[i].name, line);
    sscanf(aux+2, "%d/%d/%d %d/%d/%d", &people[i].birth.day, 
                  &people[i].birth.month, &people[i].birth.year,
                  &people[i].death.day, &people[i].death.month,
                  &people[i].death.year);
    people[i].life_days = date_difference(people[i].death, people[i].birth);
    i++;
  }
  fclose(fp);

  return i;
}

/*
 *  sorting based on the life duration
 */
void sort_array(person_t people[], int num_person)
{
  int i, j;
  person_t current;

  for (i=1; i<num_person; i++) {
    current = people[i];
    j = i;
    while ((--j>=0) && (current.life_days<people[j].life_days)) {
      people[j+1] = people[j];
    }
    people[j+1] = current;
  }
}

/*
 *  write the output file 
 */
void write_output(person_t people[], int num_person)
{
  int i;
  FILE *fp;

  fp = fopen("lifelength.txt", "w");
  if (fp == NULL) {
    printf("Error opening the output file.\n");
    exit(EXIT_FAILURE);
  }

  for (i=0; i<num_person; i++) {
    fprintf(fp, "%-20s  ", people[i].name);
    fprintf(fp, "%02d/%02d/%04d  ", people[i].birth.day, 
                 people[i].birth.month, people[i].birth.year);
    fprintf(fp, "%02d/%02d/%04d  -  ", people[i].death.day, 
                 people[i].death.month, people[i].death.year);
    fprintf(fp, "%5d\n", people[i].life_days);
  }
  fclose(fp);
}

/*
 *  compute the difference between two dates (in days)
 */
int date_difference(date_t end, date_t start)
{
  int days, i;

  /* years */
  days = (end.year - start.year) * 365;
  if (end.month < start.month) {
    days -= 365;
  }

  /* months */
  for (i=start.month-1; i!=end.month-1; i=(i+1)%12) {
    days += month_days[i];
  }

  /* days */
  days += end.day - start.day + 1;

  return days;
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
