/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 * Capitolo 11 - Esercizio Proposto 06 - Soluzione 1
 *
 * Nessuna garanzia viene fornita su questo programma:
 * gli Autori e la Casa Editrice non potranno in nessun
 * caso essere ritenuti responsabili di eventuali danni
 * (materiali e/o immateriali) da esso cagionati.
 * E' liberamente consentita la modifica (ma non la
 * redistribuzione) di questo file, a patto che le note
 * di copyright poste in testa e in coda a esso vengano
 * mantenute inalterate.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_RECTANGLE 100
#define MAX_CODE        4

/* structure declaration */
typedef struct {
  float x;
  float y;
} point_t;

typedef struct {
  char code[MAX_CODE+1];
  point_t p1;
  point_t p2;
  float area;
} rectangle_t;

/* function prototypes */
int rectangle_input(char *, rectangle_t []);
void rectangle_sort(rectangle_t [], int);
void rectangle_print(rectangle_t [], int);

/*
 *  main program
 */
int main(int argc, char *argv[])
{
  rectangle_t rct[MAX_RECTANGLE];
  int rectangle_num;

  if (argc < 2) {
    printf("Error: missing parameter.\n");
    printf("Run as: %s <input_file>.\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  rectangle_num = rectangle_input(argv[1], rct);
  rectangle_sort(rct, rectangle_num);
  rectangle_print(rct, rectangle_num);
  return EXIT_SUCCESS;
}

/*
 *  load rectangles
 */
int rectangle_input(char *fname, rectangle_t rct[MAX_RECTANGLE])
{
  int i, found, num_rct=0;
  char name[20];
  float x, y;
  FILE *fin;

  fin = fopen(fname, "r");
  if (fin == NULL) {
    printf("Error: input file not readable.\n");
    exit(EXIT_FAILURE);
  }

  /* data structure initialization */
  for (i=0; i<MAX_RECTANGLE; i++) {
    rct[i].code[0] = '\0';
    rct[i].area = 0;
    rct[i].p1.x = rct[i].p1.y = 0;
    rct[i].p2.x = rct[i].p2.y = 0;
  }

  while (fscanf(fin, "%s%f%f", name, &x, &y) != EOF) {
    found = 0;
    for (i=0; i<num_rct && !found; i++) {
      if (strcmp(name, rct[i].code) == 0) {
        /* rectangle found: second point */
        rct[i].p2.x = x;
        rct[i].p2.y = y;
        rct[i].area = (x - rct[i].p1.x) * (y - rct[i].p1.y);
        if (rct[i].area < 0) {
          rct[i].area = -rct[i].area;
        }
        found = 1;
      }
    }
    if (!found) {
      /* rectangle NOT found: first point */
      strcpy(rct[i].code, name);
      rct[i].p1.x = x;
      rct[i].p1.y = y;
      num_rct++;
    }
  }
  fclose(fin);

  return num_rct;
}

/*
 *  sort rectangles (by area)
 */
void rectangle_sort(rectangle_t rct[MAX_RECTANGLE], int num_rct)
{
  rectangle_t temp;
  int i, j, pos;

  for (i=0; i<num_rct-1; i++) {
    for (pos=i, j=i+1; j<num_rct; j++) {
      if (rct[j].area > rct[pos].area) {
        pos = j;
      }
    }
    temp = rct[pos];
    rct[pos] = rct[i];
    rct[i] = temp;
  }
}

/*
 *  display result 
 */
void rectangle_print(rectangle_t rct[MAX_RECTANGLE], int num_rct)
{
  int i;

  for (i=0; i<num_rct; i++) {
    printf("Rectangle %s: area = %f\n", rct[i].code, rct[i].area);
  }
}

/*
 * Copyright C.L.U.T. - Torino - Ottobre 2016
 *
 * Sergio Nocco   Stefano Quer
 * Guida alla Programmazione in Linguaggio C
 * Volume I: Fondamenti di programmazione
 *
 */
